/*@lineinfo:filename=FundamentalExample3*//*@lineinfo:user-code*//*@lineinfo:1^1*//*
   The program FundamentalExample3 illustrates the use of a positional
   iterator to display the id, name, description and price columns
   from the products table.
*/

// import required packages
import java.sql.*;
import oracle.sqlj.runtime.Oracle;

public class FundamentalExample3 {

  // step 1: declare the iterator class
  /*@lineinfo:generated-code*//*@lineinfo:14^3*/

//  ************************************************************
//  SQLJ iterator declaration:
//  ************************************************************

private static class ProductIteratorPosClass 
extends sqlj.runtime.ref.ResultSetIterImpl
implements sqlj.runtime.PositionedIterator
{
  public ProductIteratorPosClass(sqlj.runtime.profile.RTResultSet resultSet) 
    throws java.sql.SQLException 
  {
    super(resultSet, 4);
  }
  public int getCol1() 
    throws java.sql.SQLException 
  {
    return resultSet.getIntNoNull(1);
  }
  public String getCol2() 
    throws java.sql.SQLException 
  {
    return resultSet.getString(2);
  }
  public String getCol3() 
    throws java.sql.SQLException 
  {
    return resultSet.getString(3);
  }
  public double getCol4() 
    throws java.sql.SQLException 
  {
    return resultSet.getDoubleNoNull(4);
  }
  public static final int my_constant = 150;
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:19^3*/

  public static void main(String [] args) throws SQLException {

    try {

      Oracle.connect(
        "jdbc:oracle:thin:@localhost:1521:orcl",
        "fundamental_user",
        "fundamental_password"
      );

      // step 2: declare an iterator object from the iterator class
      ProductIteratorPosClass product_iterator;

      // step 3: populate the iterator object using a SELECT statement
      /*@lineinfo:generated-code*//*@lineinfo:35^7*/

//  ************************************************************
//  #sql product_iterator = { SELECT
//            id, name, description, price
//          FROM
//            products
//          WHERE
//            id <= 5
//         };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, FundamentalExample3_SJProfileKeys.getKey(0), 0);
    try 
    {
      sqlj.runtime.profile.RTResultSet __sJT_result = __sJT_execCtx.executeQuery();
      product_iterator = new ProductIteratorPosClass(__sJT_result);
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:42^7*/

      System.out.println("Product details:");
      System.out.println(product_iterator.my_constant);

      // declare the host variables that will be used to store
      // the columns contained in the iterator object
      boolean finished = false;
      int id = 0;
      String name = null;
      String description = null;
      double price = 0.0;

      // step 4: read the rows from the iterator object
      while (!finished) {

        // each time the FETCH statement is used on the iterator object,
        // the next row in the iterator object will be read
        /*@lineinfo:generated-code*//*@lineinfo:60^9*/

//  ************************************************************
//  #sql { FETCH :product_iterator   };
//  ************************************************************

{
  ProductIteratorPosClass __sJT_iter = product_iterator;
  if (__sJT_iter.next())
  {
    id = __sJT_iter.getCol1();
    name = __sJT_iter.getCol2();
    description = __sJT_iter.getCol3();
    price = __sJT_iter.getCol4();
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:60^78*/

        // after the last row has been read, the endFetch() method will
        // return the boolean true value (this indicates that all the rows
        // in the iterator object have been read)
        if (product_iterator.endFetch()) {

          finished = true;

        } else {

          // display the values
          System.out.println("id = " + id);
          System.out.println("name = " + name);
          System.out.println("description = " + description);
          System.out.println("price = " + price);

        }

      } // end of while loop


      // step 5: close the iterator object
      product_iterator.close();

      Oracle.close();

    } catch ( SQLException e ) {

      System.err.println("SQLException " + e);
      System.exit(1);

    }

  } // end of main()

}/*@lineinfo:generated-code*/class FundamentalExample3_SJProfileKeys 
{
  private static FundamentalExample3_SJProfileKeys inst = null;
  public static java.lang.Object getKey(int keyNum) 
    throws java.sql.SQLException 
  {
    if (inst == null)
    {
      inst = new FundamentalExample3_SJProfileKeys();
    }
    return inst.keys[keyNum];
  }
  private final sqlj.runtime.profile.Loader loader = sqlj.runtime.RuntimeContext.getRuntime().getLoaderForClass(getClass());
  private java.lang.Object[] keys;
  private FundamentalExample3_SJProfileKeys() 
    throws java.sql.SQLException 
  {
    keys = new java.lang.Object[1];
    keys[0] = sqlj.runtime.ref.DefaultContext.getProfileKey(loader, "FundamentalExample3_SJProfile0");
  }
}
