/*@lineinfo:filename=LobExample1*//*@lineinfo:user-code*//*@lineinfo:1^1*//*
   The program LobExample1.sqlj illustrates how to
   use the large object types CLOB, BLOB and BFILE
   to store text and images in the database.
*/

import java.sql.*;
import oracle.sqlj.runtime.Oracle;
import java.io.*;
import sqlj.runtime.*;
import oracle.sql.*;

public class LobExample1 {

  public static void main(String [] args) {

    try {

      Oracle.connect(
        "jdbc:oracle:thin:@localhost:1521:orcl",
        "lob_user",
        "lob_password"
      );

      String source_directory = "C:\\sample_files\\";

      addCLOB(source_directory + "textContent.txt");
      addBLOB(source_directory + "imageContent.jpg");
      addBFILE("SAMPLE_FILES_DIR", "textContent.txt");
      addBFILE("SAMPLE_FILES_DIR", "imageContent.jpg");

      Oracle.close();

    } catch (IOException e) {

      System.err.println("IOException " + e);
      System.exit(1);

    } catch (SQLException e) {

      System.err.println("SQLException " + e);
      System.exit(1);

    }

  } // end of main()


  private static void addCLOB(String file_name)
  throws IOException, SQLException {

    // step 1: initialize the LOB column to set the LOB locator
    /*@lineinfo:generated-code*//*@lineinfo:53^5*/

//  ************************************************************
//  #sql { INSERT INTO
//          clob_content(
//            file_name,
//            clob_column
//          ) VALUES (
//            :file_name,
//            EMPTY_CLOB()
//          )
//       };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  String __sJT_1 = file_name;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, LobExample1_SJProfileKeys.getKey(0), 0);
    try 
    {
      __sJT_stmt.setString(1, __sJT_1);
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:62^5*/

    // step 2: declare a host object of the LOB type
    CLOB host_clob;

    // step 3: retrieve the LOB locator into the LOB host object
    /*@lineinfo:generated-code*//*@lineinfo:68^5*/

//  ************************************************************
//  #sql { SELECT
//          clob_column
//        
//        FROM
//          clob_content
//        WHERE
//          file_name = :file_name
//        FOR UPDATE
//       };
//  ************************************************************

{
  sqlj.runtime.profile.RTResultSet __sJT_rtRs;
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  String __sJT_1 = file_name;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, LobExample1_SJProfileKeys.getKey(0), 1);
    try 
    {
      __sJT_stmt.setString(1, __sJT_1);
      sqlj.runtime.profile.RTResultSet __sJT_result = __sJT_execCtx.executeQuery();
      __sJT_rtRs = __sJT_result;
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
  try 
  {
    sqlj.runtime.ref.ResultSetIterImpl.checkColumns(__sJT_rtRs, 1);
    if (!__sJT_rtRs.next())
    {
      sqlj.runtime.error.RuntimeRefErrors.raise_NO_ROW_SELECT_INTO();
    }
    host_clob = ((CLOB)(__sJT_rtRs.getObject(1, CLOB.class)));
    if (__sJT_rtRs.next())
    {
      sqlj.runtime.error.RuntimeRefErrors.raise_MULTI_ROW_SELECT_INTO();
    }
  }
  finally 
  {
    __sJT_rtRs.close();
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:78^5*/

    // step 4: get the chunk size of the LOB from the LOB host object
    int size_of_chunk = host_clob.getChunkSize();

    // step 5: create a buffer to hold a block of data from the file
    char [] text_buffer = new char[size_of_chunk];

    // step 6: create a file object
    File file = new File(file_name);

    // step 7: create input stream objects to read the file contents
    FileInputStream file_input_stream = new FileInputStream(file);
    InputStreamReader reader = new InputStreamReader(file_input_stream);
    BufferedReader buffered_reader = new BufferedReader(reader);

    // step 8: read the file contents and write it to the LOB
    long position = 1;
    int chars_read;

    while ((chars_read = buffered_reader.read(text_buffer)) != -1) {

      // write the buffer contents to host_clob using the putChars() method
      host_clob.putChars(position, text_buffer);

      // increment the end position
      position += chars_read;

    } // end of while

    // step 9: perform a COMMIT
    /*@lineinfo:generated-code*//*@lineinfo:109^5*/

//  ************************************************************
//  #sql { COMMIT  };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, LobExample1_SJProfileKeys.getKey(0), 2);
    try 
    {
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:109^19*/

    // step 10: close the objects used to read the file
    buffered_reader.close();
    reader.close();
    file_input_stream.close();

    System.out.println("Successfully added content from file " +
      file_name + " to CLOB");

  } // end of addCLOB()


  private static void addBLOB(String file_name)
  throws IOException, SQLException {

    // step 1: initialize the LOB column to set the LOB locator
    /*@lineinfo:generated-code*//*@lineinfo:126^5*/

//  ************************************************************
//  #sql { INSERT INTO
//          blob_content(
//            file_name,
//            blob_column
//          ) VALUES (
//            :file_name,
//            EMPTY_BLOB()
//          )
//       };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  String __sJT_1 = file_name;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, LobExample1_SJProfileKeys.getKey(0), 3);
    try 
    {
      __sJT_stmt.setString(1, __sJT_1);
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:135^5*/

    // step 2: declare a host object of the LOB type
    BLOB host_blob;

    // step 3: retrieve the LOB locator into the LOB host object
    /*@lineinfo:generated-code*//*@lineinfo:141^5*/

//  ************************************************************
//  #sql { SELECT
//          blob_column
//        
//        FROM
//          blob_content
//        WHERE
//          file_name = :file_name
//        FOR UPDATE
//       };
//  ************************************************************

{
  sqlj.runtime.profile.RTResultSet __sJT_rtRs;
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  String __sJT_1 = file_name;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, LobExample1_SJProfileKeys.getKey(0), 4);
    try 
    {
      __sJT_stmt.setString(1, __sJT_1);
      sqlj.runtime.profile.RTResultSet __sJT_result = __sJT_execCtx.executeQuery();
      __sJT_rtRs = __sJT_result;
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
  try 
  {
    sqlj.runtime.ref.ResultSetIterImpl.checkColumns(__sJT_rtRs, 1);
    if (!__sJT_rtRs.next())
    {
      sqlj.runtime.error.RuntimeRefErrors.raise_NO_ROW_SELECT_INTO();
    }
    host_blob = ((BLOB)(__sJT_rtRs.getObject(1, BLOB.class)));
    if (__sJT_rtRs.next())
    {
      sqlj.runtime.error.RuntimeRefErrors.raise_MULTI_ROW_SELECT_INTO();
    }
  }
  finally 
  {
    __sJT_rtRs.close();
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:151^5*/

    // step 4: get the chunk size of the LOB from the LOB host object
    int size_of_chunk = host_blob.getChunkSize();

    // step 5: create a buffer to hold a block of data from the file
    byte [] byte_buffer = new byte[size_of_chunk];

    // step 6: create a file object to open the file
    File file = new File(file_name);

    // step 7: create an input stream object to read the file contents
    FileInputStream file_input_stream = new FileInputStream(file);

    // step 8: read the file contents and write it to the LOB
    long position = 1;
    int bytes_read;

    while ((bytes_read = file_input_stream.read(byte_buffer)) != -1) {

      // copy the buffer contents to host_blob using the putBytes() method
      host_blob.putBytes(position, byte_buffer);

      // increment the end position
      position += bytes_read;

    } // end of while

    // step 9: perform a COMMIT
    /*@lineinfo:generated-code*//*@lineinfo:180^5*/

//  ************************************************************
//  #sql { COMMIT  };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, LobExample1_SJProfileKeys.getKey(0), 5);
    try 
    {
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:180^19*/

    // step 10: close the objects used to read the file
    file_input_stream.close();

    System.out.println("Successfully added content from file " +
      file_name + " to BLOB");

  } // end of addBLOB()


  private static void addBFILE(
    String directory,
    String file_name
  ) throws SQLException {

    /*@lineinfo:generated-code*//*@lineinfo:196^5*/

//  ************************************************************
//  #sql { INSERT INTO
//          bfile_content(
//            file_name,
//            bfile_column
//          ) VALUES (
//            :file_name,
//            BFILENAME(:directory, :file_name)
//          )
//       };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  String __sJT_1 = file_name;
  String __sJT_2 = directory;
  String __sJT_3 = file_name;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, LobExample1_SJProfileKeys.getKey(0), 6);
    try 
    {
      __sJT_stmt.setString(1, __sJT_1);
      __sJT_stmt.setString(2, __sJT_2);
      __sJT_stmt.setString(3, __sJT_3);
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:205^5*/

    /*@lineinfo:generated-code*//*@lineinfo:207^5*/

//  ************************************************************
//  #sql { COMMIT  };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, LobExample1_SJProfileKeys.getKey(0), 7);
    try 
    {
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:207^19*/

    System.out.println("Successfully added pointer to file " +
      file_name + " to BFILE in database directory " + directory);

  } // end of addBFILE()

}/*@lineinfo:generated-code*/class LobExample1_SJProfileKeys 
{
  private static LobExample1_SJProfileKeys inst = null;
  public static java.lang.Object getKey(int keyNum) 
    throws java.sql.SQLException 
  {
    if (inst == null)
    {
      inst = new LobExample1_SJProfileKeys();
    }
    return inst.keys[keyNum];
  }
  private final sqlj.runtime.profile.Loader loader = sqlj.runtime.RuntimeContext.getRuntime().getLoaderForClass(getClass());
  private java.lang.Object[] keys;
  private LobExample1_SJProfileKeys() 
    throws java.sql.SQLException 
  {
    keys = new java.lang.Object[1];
    keys[0] = sqlj.runtime.ref.DefaultContext.getProfileKey(loader, "LobExample1_SJProfile0");
  }
}
