/*@lineinfo:filename=LobExample3*//*@lineinfo:user-code*//*@lineinfo:1^1*//*
   The program LobExample3.sqlj illustrates how to
   use the large object types CLOB, BLOB and BFILE
   to store text and images in the database using
   streams.
*/

import java.sql.*;
import oracle.sqlj.runtime.Oracle;
import java.io.*;
import sqlj.runtime.*;
import oracle.sql.*;

public class LobExample3 {

  public static void main(String [] args) {

    try {

      Oracle.connect(
        "jdbc:oracle:thin:@localhost:1521:orcl",
        "lob_user",
        "lob_password"
      );

      String source_directory = "C:\\sample_files\\";

      addCLOB(source_directory + "textContent.txt");
      addBLOB(source_directory + "imageContent.jpg");

      Oracle.close();

    } catch (IOException e) {

      System.err.println("IOException " + e);
      System.exit(1);

    } catch (SQLException e) {

      System.err.println("SQLException " + e);
      System.exit(1);

    }

  } // end of main()


  private static void addCLOB(String file_name)
  throws IOException, SQLException {

    // step 1: initialize the LOB column to set the locator
    /*@lineinfo:generated-code*//*@lineinfo:52^5*/

//  ************************************************************
//  #sql { INSERT INTO
//          clob_content(
//            file_name,
//            clob_column
//          ) VALUES (
//            :file_name,
//            EMPTY_CLOB()
//          )
//       };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  String __sJT_1 = file_name;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, LobExample3_SJProfileKeys.getKey(0), 0);
    try 
    {
      __sJT_stmt.setString(1, __sJT_1);
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:61^5*/

    // step 2: declare a host object of the LOB type
    CLOB host_clob;

    // step 3: retrieve the LOB locator into the LOB host object
    /*@lineinfo:generated-code*//*@lineinfo:67^5*/

//  ************************************************************
//  #sql { SELECT
//          clob_column
//        
//        FROM
//          clob_content
//        WHERE
//          file_name = :file_name
//        FOR UPDATE
//       };
//  ************************************************************

{
  sqlj.runtime.profile.RTResultSet __sJT_rtRs;
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  String __sJT_1 = file_name;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, LobExample3_SJProfileKeys.getKey(0), 1);
    try 
    {
      __sJT_stmt.setString(1, __sJT_1);
      sqlj.runtime.profile.RTResultSet __sJT_result = __sJT_execCtx.executeQuery();
      __sJT_rtRs = __sJT_result;
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
  try 
  {
    sqlj.runtime.ref.ResultSetIterImpl.checkColumns(__sJT_rtRs, 1);
    if (!__sJT_rtRs.next())
    {
      sqlj.runtime.error.RuntimeRefErrors.raise_NO_ROW_SELECT_INTO();
    }
    host_clob = ((CLOB)(__sJT_rtRs.getObject(1, CLOB.class)));
    if (__sJT_rtRs.next())
    {
      sqlj.runtime.error.RuntimeRefErrors.raise_MULTI_ROW_SELECT_INTO();
    }
  }
  finally 
  {
    __sJT_rtRs.close();
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:77^5*/

    // step 4: get the buffer size of the LOB from the LOB host object
    int buffer_size = host_clob.getBufferSize();

    // step 5: create a buffer to hold a block of data from the file
    byte [] byte_buffer = new byte[buffer_size];

    // step 6: create a file object to open the file
    File file = new File(file_name);

    // step 7: create a file input stream object to read the file contents
    FileInputStream file_input_stream = new FileInputStream(file);

    // step 8: create an input stream object and call the appropriate
    // LOB host object output stream function
    OutputStream output_stream = host_clob.getAsciiOutputStream();

    // step 9: while the end of the file has not been reached,
    // read a block from the file into the buffer, and write the
    // buffer contents to the LOB host object via the output stream
    int bytes_read;

    while ((bytes_read = file_input_stream.read(byte_buffer)) != -1) {

      // write the buffer contents to the LOB output stream
      // using the write() method in the output stream
      output_stream.write(byte_buffer);

    } // end of while

    // step 10: close the stream objects
    file_input_stream.close();
    output_stream.close();

    System.out.println("Successfully added content from file " +
      file_name + " to CLOB");

  } // end of addCLOB()


  private static void addBLOB(String file_name)
  throws IOException, SQLException {

    // step 1: initialize the LOB column to set the locator
    /*@lineinfo:generated-code*//*@lineinfo:122^5*/

//  ************************************************************
//  #sql { INSERT INTO
//          blob_content(
//            file_name,
//            blob_column
//          ) VALUES (
//            :file_name,
//            EMPTY_BLOB()
//          )
//       };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  String __sJT_1 = file_name;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, LobExample3_SJProfileKeys.getKey(0), 2);
    try 
    {
      __sJT_stmt.setString(1, __sJT_1);
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:131^5*/

    // step 2: declare a host object of the LOB type
    BLOB host_blob;

    // step 3: retrieve the LOB locator into the LOB host object
    /*@lineinfo:generated-code*//*@lineinfo:137^5*/

//  ************************************************************
//  #sql { SELECT
//          blob_column
//        
//        FROM
//          blob_content
//        WHERE
//          file_name = :file_name
//        FOR UPDATE
//       };
//  ************************************************************

{
  sqlj.runtime.profile.RTResultSet __sJT_rtRs;
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  String __sJT_1 = file_name;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, LobExample3_SJProfileKeys.getKey(0), 3);
    try 
    {
      __sJT_stmt.setString(1, __sJT_1);
      sqlj.runtime.profile.RTResultSet __sJT_result = __sJT_execCtx.executeQuery();
      __sJT_rtRs = __sJT_result;
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
  try 
  {
    sqlj.runtime.ref.ResultSetIterImpl.checkColumns(__sJT_rtRs, 1);
    if (!__sJT_rtRs.next())
    {
      sqlj.runtime.error.RuntimeRefErrors.raise_NO_ROW_SELECT_INTO();
    }
    host_blob = ((BLOB)(__sJT_rtRs.getObject(1, BLOB.class)));
    if (__sJT_rtRs.next())
    {
      sqlj.runtime.error.RuntimeRefErrors.raise_MULTI_ROW_SELECT_INTO();
    }
  }
  finally 
  {
    __sJT_rtRs.close();
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:147^5*/

    // step 4: get the buffer size of the LOB from the LOB host object
    int buffer_size = host_blob.getBufferSize();

    // step 5: create a buffer to hold a block of data from the file
    byte [] byte_buffer = new byte[buffer_size];

    // step 6: create a file object to open the file
    File file = new File(file_name);

    // step 7: create a file input stream object to read the file contents
    FileInputStream file_input_stream = new FileInputStream(file);

    // step 8: create an output stream object and call the appropriate
    // LOB host object output stream function
    OutputStream output_stream = host_blob.getBinaryOutputStream();

    // step 9: while the end of the file has not been reached,
    // read a block from the file into the buffer, and write the
    // buffer contents to the LOB host object via the output stream
    int bytes_read;

    while ((bytes_read = file_input_stream.read(byte_buffer)) != -1) {

      // write the buffer contents to the LOB output stream
      // using the write() method in the output stream
      output_stream.write(byte_buffer);

    } // end of while

    // step 10: close the stream objects
    file_input_stream.close();
    output_stream.close();

    System.out.println("Successfully added content from file " +
      file_name + " to BLOB");

  } // end of addBLOB()

}/*@lineinfo:generated-code*/class LobExample3_SJProfileKeys 
{
  private static LobExample3_SJProfileKeys inst = null;
  public static java.lang.Object getKey(int keyNum) 
    throws java.sql.SQLException 
  {
    if (inst == null)
    {
      inst = new LobExample3_SJProfileKeys();
    }
    return inst.keys[keyNum];
  }
  private final sqlj.runtime.profile.Loader loader = sqlj.runtime.RuntimeContext.getRuntime().getLoaderForClass(getClass());
  private java.lang.Object[] keys;
  private LobExample3_SJProfileKeys() 
    throws java.sql.SQLException 
  {
    keys = new java.lang.Object[1];
    keys[0] = sqlj.runtime.ref.DefaultContext.getProfileKey(loader, "LobExample3_SJProfile0");
  }
}
