/*@lineinfo:filename=LobExample4*//*@lineinfo:user-code*//*@lineinfo:1^1*//*
   The program LobExample4.sqlj illustrates how to
   retrieve content from the large object types
   CLOB, BLOB and BFILE using streams.
*/

import java.sql.*;
import oracle.sqlj.runtime.Oracle;
import java.io.*;
import sqlj.runtime.*;
import oracle.sql.*;

public class LobExample4 {

  public static void main(String [] args) {

    try {

      Oracle.connect(
        "jdbc:oracle:thin:@localhost:1521:orcl",
        "lob_user",
        "lob_password"
      );

      String source_directory = "C:\\sample_files\\";
      String target_directory = "C:\\sample_files\\retrieved\\";

      retrieveCLOB("textContent.txt", source_directory, target_directory);
      retrieveBLOB("imageContent.jpg", source_directory, target_directory);

      Oracle.close();

    } catch (IOException e) {

      System.err.println("IOException " + e);
      System.exit(1);

    } catch (SQLException e) {

      System.err.println("SQLException " + e);
      System.exit(1);

    }

  }


  private static void retrieveCLOB(
    String file_name,
    String source_directory,
    String target_directory
  ) throws IOException, SQLException {

    // step 1: declare a host object of the LOB type
    CLOB host_clob;

    // step 2: retrieve the LOB locator into the LOB host object
    /*@lineinfo:generated-code*//*@lineinfo:58^5*/

//  ************************************************************
//  #sql { SELECT
//          clob_column
//        
//        FROM
//          clob_content
//        WHERE
//          file_name = :source_directory + file_name
//       };
//  ************************************************************

{
  sqlj.runtime.profile.RTResultSet __sJT_rtRs;
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  String __sJT_1 = source_directory + file_name;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, LobExample4_SJProfileKeys.getKey(0), 0);
    try 
    {
      __sJT_stmt.setString(1, __sJT_1);
      sqlj.runtime.profile.RTResultSet __sJT_result = __sJT_execCtx.executeQuery();
      __sJT_rtRs = __sJT_result;
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
  try 
  {
    sqlj.runtime.ref.ResultSetIterImpl.checkColumns(__sJT_rtRs, 1);
    if (!__sJT_rtRs.next())
    {
      sqlj.runtime.error.RuntimeRefErrors.raise_NO_ROW_SELECT_INTO();
    }
    host_clob = ((CLOB)(__sJT_rtRs.getObject(1, CLOB.class)));
    if (__sJT_rtRs.next())
    {
      sqlj.runtime.error.RuntimeRefErrors.raise_MULTI_ROW_SELECT_INTO();
    }
  }
  finally 
  {
    __sJT_rtRs.close();
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:67^5*/

    // step 3: create an input stream object and call the appropriate
    // LOB host object input stream function, in preparation for reading 
    // the contents of the LOB
    InputStream input_stream = host_clob.getAsciiStream();

    // step 4: save the contents of the LOB to a new file,
    // reading the LOB contents from the input stream object
    String save_file_name = target_directory + "retrievedCLOB2" + file_name;
    saveFile(input_stream, save_file_name);

    // step 5: close the input stream object
    input_stream.close();

    System.out.println("Successfully retrieved CLOB and saved file " +
      save_file_name);

  } // end of retrieveCLOB()


  private static void retrieveBLOB(
    String file_name,
    String source_directory,
    String target_directory
  ) throws IOException, SQLException {

    // step 1: declare a host object of the LOB type
    BLOB host_blob;

    // step 2: retrieve the LOB locator into the LOB host object
    /*@lineinfo:generated-code*//*@lineinfo:98^5*/

//  ************************************************************
//  #sql { SELECT
//          blob_column
//        
//        FROM
//          blob_content
//        WHERE
//          file_name = :source_directory + file_name
//       };
//  ************************************************************

{
  sqlj.runtime.profile.RTResultSet __sJT_rtRs;
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  String __sJT_1 = source_directory + file_name;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, LobExample4_SJProfileKeys.getKey(0), 1);
    try 
    {
      __sJT_stmt.setString(1, __sJT_1);
      sqlj.runtime.profile.RTResultSet __sJT_result = __sJT_execCtx.executeQuery();
      __sJT_rtRs = __sJT_result;
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
  try 
  {
    sqlj.runtime.ref.ResultSetIterImpl.checkColumns(__sJT_rtRs, 1);
    if (!__sJT_rtRs.next())
    {
      sqlj.runtime.error.RuntimeRefErrors.raise_NO_ROW_SELECT_INTO();
    }
    host_blob = ((BLOB)(__sJT_rtRs.getObject(1, BLOB.class)));
    if (__sJT_rtRs.next())
    {
      sqlj.runtime.error.RuntimeRefErrors.raise_MULTI_ROW_SELECT_INTO();
    }
  }
  finally 
  {
    __sJT_rtRs.close();
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:107^5*/

    // step 3: create an input stream object and call the appropriate
    // LOB host object input stream function, in preparation for reading 
    // the contents of the LOB
    InputStream input_stream = host_blob.getBinaryStream();

    // step 4: save the contents of the LOB to a new file,
    // reading the LOB contents from the input stream object
    String save_file_name = target_directory + "retrievedBLOB2" + file_name;
    saveFile(input_stream, save_file_name);

    // step 5: close the input stream object
    input_stream.close();

    System.out.println("Successfully retrieved BLOB and saved file " +
      save_file_name);

  } // end of retrieveBLOB()


  private static void saveFile(
    InputStream input_stream,
    String file_name
  ) throws IOException {

    // create a file object
    File file = new File(file_name);

    // create a file output stream
    FileOutputStream file_output_stream = new FileOutputStream(file);

    // read the contents of the input stream until
    // the end of the input stream has been reached (indicated
    // by -1 being returned from the read() method)
    byte [] byte_buffer = new byte[8132];
    int bytes_read;
    while ((bytes_read = input_stream.read(byte_buffer)) != -1 ){

      // write the input to the file
      file_output_stream.write(byte_buffer);

    } // end of while

    // close the file output stream
    file_output_stream.close();

  } // end of saveFile()

}/*@lineinfo:generated-code*/class LobExample4_SJProfileKeys 
{
  private static LobExample4_SJProfileKeys inst = null;
  public static java.lang.Object getKey(int keyNum) 
    throws java.sql.SQLException 
  {
    if (inst == null)
    {
      inst = new LobExample4_SJProfileKeys();
    }
    return inst.keys[keyNum];
  }
  private final sqlj.runtime.profile.Loader loader = sqlj.runtime.RuntimeContext.getRuntime().getLoaderForClass(getClass());
  private java.lang.Object[] keys;
  private LobExample4_SJProfileKeys() 
    throws java.sql.SQLException 
  {
    keys = new java.lang.Object[1];
    keys[0] = sqlj.runtime.ref.DefaultContext.getProfileKey(loader, "LobExample4_SJProfile0");
  }
}
