/*@lineinfo:filename=PerformanceExample2*//*@lineinfo:user-code*//*@lineinfo:1^1*//*
   The program PerformanceExample2.sqlj illustrates how to use
   SQL statement batching to improve performance.
*/

import java.sql.*;
import oracle.sqlj.runtime.Oracle;
import sqlj.runtime.ref.DefaultContext;
import oracle.jdbc.driver.OracleConnection;
import sqlj.runtime.ExecutionContext;

public class PerformanceExample2 {

  public static final int ROWS = 2000;
  public static final int BATCH_SIZE = 10;

  public static void main(String [] args) {

    try {

      // connect to a database on a remote machine
      Oracle.connect(
        "jdbc:oracle:thin:@remotehost:1521:orcl",
        "fundamental_user",
        "fundamental_password"
      );

      // create a temporary table
      /*@lineinfo:generated-code*//*@lineinfo:29^7*/

//  ************************************************************
//  #sql { CREATE TABLE batch_table (
//            value INTEGER
//          )
//         };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, PerformanceExample2_SJProfileKeys.getKey(0), 0);
    try 
    {
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:33^7*/

      // display the status of batching
      boolean is_batching = DefaultContext.getDefaultContext().
        getExecutionContext().isBatching();
      System.out.println("is_batching = " + is_batching);

      insertRows();

      // set batching to true for the default execution context
      DefaultContext.getDefaultContext().getExecutionContext().
        setBatching(true);

      // display the status of batching
      is_batching = DefaultContext.getDefaultContext().
        getExecutionContext().isBatching();
      System.out.println("is_batching = " + is_batching);

      // set the batch limit to BATCH_SIZE and display the batch limit
      System.out.println("Setting the batch limit to " + BATCH_SIZE + ".");
      DefaultContext.getDefaultContext().
        getExecutionContext().setBatchLimit(BATCH_SIZE);
      int batch_limit = DefaultContext.getDefaultContext().
        getExecutionContext().getBatchLimit();
      System.out.println("batch_limit = " + batch_limit);

      insertRows();

      // explicitly execute the last batch
      int [] batch_results = DefaultContext.getDefaultContext().
        getExecutionContext().executeBatch();
      System.out.println("batch_results.length = " + batch_results.length);
      for (int count = 0; count < batch_results.length; count++) {
        System.out.println("batch_results[" + count + "] = " +
          batch_results[count]);
      }

      // drop the batch table
      /*@lineinfo:generated-code*//*@lineinfo:71^7*/

//  ************************************************************
//  #sql { DROP TABLE batch_table  };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, PerformanceExample2_SJProfileKeys.getKey(0), 1);
    try 
    {
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:71^37*/

      Oracle.close();

    } catch (SQLException e) {

      System.err.println("SQLException " + e);
      System.exit(1);

    }

  } // end of main()


  private static void insertRows()
  throws SQLException {

    long start_time = System.currentTimeMillis();

    for (int count = 0; count < ROWS; count++) {
      /*@lineinfo:generated-code*//*@lineinfo:91^7*/

//  ************************************************************
//  #sql { INSERT INTO batch_table
//            (value)
//          VALUES
//            (:count)
//         };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = sqlj.runtime.ref.DefaultContext.getDefaultContext();
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  int __sJT_1 = count;
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, PerformanceExample2_SJProfileKeys.getKey(0), 2);
    try 
    {
      __sJT_stmt.setInt(1, __sJT_1);
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:96^7*/
    }

    long end_time = System.currentTimeMillis();
    System.out.println("Total time for inserting " + ROWS +
      " rows was " + (end_time - start_time) + " milliseconds.");

  } // end of insertRows()

}/*@lineinfo:generated-code*/class PerformanceExample2_SJProfileKeys 
{
  private static PerformanceExample2_SJProfileKeys inst = null;
  public static java.lang.Object getKey(int keyNum) 
    throws java.sql.SQLException 
  {
    if (inst == null)
    {
      inst = new PerformanceExample2_SJProfileKeys();
    }
    return inst.keys[keyNum];
  }
  private final sqlj.runtime.profile.Loader loader = sqlj.runtime.RuntimeContext.getRuntime().getLoaderForClass(getClass());
  private java.lang.Object[] keys;
  private PerformanceExample2_SJProfileKeys() 
    throws java.sql.SQLException 
  {
    keys = new java.lang.Object[1];
    keys[0] = sqlj.runtime.ref.DefaultContext.getProfileKey(loader, "PerformanceExample2_SJProfile0");
  }
}
