/*@lineinfo:filename=TransactionExample1*//*@lineinfo:user-code*//*@lineinfo:1^1*//*
   The program TransactionExample1.sqlj illustrates the use of the
   SQL SET TRANSACTION statement to set the transaction isolation level.
*/

import java.sql.*;
import oracle.sqlj.runtime.Oracle;
import sqlj.runtime.ref.DefaultContext;

public class TransactionExample1 {

  /*@lineinfo:generated-code*//*@lineinfo:12^3*/

//  ************************************************************
//  SQLJ iterator declaration:
//  ************************************************************

private static class CustomerIteratorClass 
extends sqlj.runtime.ref.ResultSetIterImpl
implements sqlj.runtime.NamedIterator
{
  public CustomerIteratorClass(sqlj.runtime.profile.RTResultSet resultSet) 
    throws java.sql.SQLException 
  {
    super(resultSet);
    idNdx = findColumn("id");
    first_nameNdx = findColumn("first_name");
    last_nameNdx = findColumn("last_name");
  }
  public int id() 
    throws java.sql.SQLException 
  {
    return resultSet.getIntNoNull(idNdx);
  }
  private int idNdx;
  public String first_name() 
    throws java.sql.SQLException 
  {
    return resultSet.getString(first_nameNdx);
  }
  private int first_nameNdx;
  public String last_name() 
    throws java.sql.SQLException 
  {
    return resultSet.getString(last_nameNdx);
  }
  private int last_nameNdx;
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:14^3*/

  public static void main(String [] args) {

    try {

      // create a connection context named mod_context
      DefaultContext mod_context = Oracle.getConnection(
        "jdbc:oracle:thin:@localhost:1521:orcl",
        "fundamental_user",
        "fundamental_password"
      );

      // create a connection context named ser_context
      DefaultContext ser_context = Oracle.getConnection(
        "jdbc:oracle:thin:@localhost:1521:orcl",
        "fundamental_user",
        "fundamental_password"
      );

      // use the SET TRANSACTION statement to set the isolation
      // level to SERIALIZABLE for ser_context,
      // ser_context will no longer be subject to phantom reads or
      // non-repeatable reads
      System.out.println("Setting the transaction isolation level to " +
        "SERIALIZABLE for ser_context.");
      /*@lineinfo:generated-code*//*@lineinfo:40^7*/

//  ************************************************************
//  #sql [ser_context] { SET TRANSACTION ISOLATION LEVEL SERIALIZABLE  };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = ser_context;
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, TransactionExample1_SJProfileKeys.getKey(0), 0);
    try 
    {
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:40^73*/

      // display all customers using mod_context
      System.out.println("List of customers using mod_context.");
      displayCustomers(mod_context);

      // display all customers using ser_context
      System.out.println("List of customers using ser_context.");
      displayCustomers(ser_context);

      // add a customer using mod_context
      System.out.println("Adding customer Jason Price using mod_context.");
      /*@lineinfo:generated-code*//*@lineinfo:52^7*/

//  ************************************************************
//  #sql [mod_context] { INSERT INTO
//            customers (id, first_name, last_name)
//          VALUES
//            (6, 'Jason', 'Price')
//         };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = mod_context;
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, TransactionExample1_SJProfileKeys.getKey(0), 1);
    try 
    {
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:57^7*/

      // update customer using mod_context
      System.out.println("Updating customer 1 name to John Doe using " +
        "mod_context.");
      /*@lineinfo:generated-code*//*@lineinfo:62^7*/

//  ************************************************************
//  #sql [mod_context] { UPDATE
//            customers
//          SET
//            first_name = 'John',
//            last_name  = 'Doe'
//          WHERE
//            id = 1
//         };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = mod_context;
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, TransactionExample1_SJProfileKeys.getKey(0), 2);
    try 
    {
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:70^7*/

      // perform a COMMIT using mod_context
      System.out.println("Performing COMMIT using mod_context.");
      /*@lineinfo:generated-code*//*@lineinfo:74^7*/

//  ************************************************************
//  #sql [mod_context] { COMMIT  };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = mod_context;
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, TransactionExample1_SJProfileKeys.getKey(0), 3);
    try 
    {
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:74^35*/

      // display all customers using mod_context
      System.out.println("List of customers using mod_context.");
      displayCustomers(mod_context);

      // display all customers using ser_context,
      // the new customer (a phantom read), and the modified customer name
      // (a non-repeatable read) are not visible to ser_context
      System.out.println("List of customers using ser_context.");
      displayCustomers(ser_context);

      // delete the new customer, change the first customer name
      // back to John Smith and perform a COMMIT using mod_context
      /*@lineinfo:generated-code*//*@lineinfo:88^7*/

//  ************************************************************
//  #sql [mod_context] { DELETE FROM
//            customers
//          WHERE
//            id = 6
//         };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = mod_context;
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, TransactionExample1_SJProfileKeys.getKey(0), 4);
    try 
    {
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:93^7*/
      /*@lineinfo:generated-code*//*@lineinfo:94^7*/

//  ************************************************************
//  #sql [mod_context] { UPDATE
//            customers
//          SET
//            first_name = 'John',
//            last_name  = 'Smith'
//          WHERE
//            id = 1
//         };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = mod_context;
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, TransactionExample1_SJProfileKeys.getKey(0), 5);
    try 
    {
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:102^7*/
      /*@lineinfo:generated-code*//*@lineinfo:103^7*/

//  ************************************************************
//  #sql [mod_context] { COMMIT  };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = mod_context;
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, TransactionExample1_SJProfileKeys.getKey(0), 6);
    try 
    {
      __sJT_execCtx.executeUpdate();
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:103^35*/

      // close the connection contexts
      mod_context.close();
      ser_context.close();

    } catch ( SQLException e ) {

      System.err.println("SQLException " + e);
      System.exit(1);

    }

  } // end of main()


  private static void displayCustomers(
    DefaultContext conn_context
  ) throws SQLException {

    CustomerIteratorClass customer_iterator;

    /*@lineinfo:generated-code*//*@lineinfo:125^5*/

//  ************************************************************
//  #sql [conn_context] customer_iterator = { SELECT
//          id, first_name, last_name
//        FROM
//          customers
//        ORDER BY
//          id
//       };
//  ************************************************************

{
  sqlj.runtime.ConnectionContext __sJT_connCtx = conn_context;
  if (__sJT_connCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_CONN_CTX();
  sqlj.runtime.ExecutionContext __sJT_execCtx = __sJT_connCtx.getExecutionContext();
  if (__sJT_execCtx == null) sqlj.runtime.error.RuntimeRefErrors.raise_NULL_EXEC_CTX();
  synchronized (__sJT_execCtx) {
    sqlj.runtime.profile.RTStatement __sJT_stmt = __sJT_execCtx.registerStatement(__sJT_connCtx, TransactionExample1_SJProfileKeys.getKey(0), 7);
    try 
    {
      sqlj.runtime.profile.RTResultSet __sJT_result = __sJT_execCtx.executeQuery();
      customer_iterator = new CustomerIteratorClass(__sJT_result);
    }
    finally 
    {
      __sJT_execCtx.releaseStatement();
    }
  }
}


//  ************************************************************

/*@lineinfo:user-code*//*@lineinfo:132^5*/

    while (customer_iterator.next()) {
      System.out.println("id = " + customer_iterator.id());
      System.out.println("first_name = " + customer_iterator.first_name());
      System.out.println("last_name = " + customer_iterator.last_name());
    } // end of while loop

    customer_iterator.close();

  } // end of displayCustomers()

}/*@lineinfo:generated-code*/class TransactionExample1_SJProfileKeys 
{
  private static TransactionExample1_SJProfileKeys inst = null;
  public static java.lang.Object getKey(int keyNum) 
    throws java.sql.SQLException 
  {
    if (inst == null)
    {
      inst = new TransactionExample1_SJProfileKeys();
    }
    return inst.keys[keyNum];
  }
  private final sqlj.runtime.profile.Loader loader = sqlj.runtime.RuntimeContext.getRuntime().getLoaderForClass(getClass());
  private java.lang.Object[] keys;
  private TransactionExample1_SJProfileKeys() 
    throws java.sql.SQLException 
  {
    keys = new java.lang.Object[1];
    keys[0] = DefaultContext.getProfileKey(loader, "TransactionExample1_SJProfile0");
  }
}
